@@ -19,7 +19,7 @@ HTML_CONTENT = """<!DOCTYPE html>
 </head>
 <body>
     <div>
-        <h1> Hello, world! </div>
+        <h1> Hello, world! </h1>
 
         <p>
             The quick red fox jumps over the lazy brown dog.
@@ -132,6 +132,29 @@ def test_document_background_image():
         tools.assert_equal(len(denker_transparant), 1)
 
 
+def test_document_background_image_not_on_all_pages():
+    """
+    Test that all pages are being rendered, when background is a pdf file and it's applied for the first page only.
+    """
+    tests_folder = os.path.dirname(os.path.realpath(__file__))
+    background_path = os.path.join(tests_folder, 'samples', 'images.pdf')
+
+    css = "<style>@page {{background-image: url({background_location});}} @page two {{}}</style>".format(
+        background_location=background_path)
+
+    extra_html = """<pdf:nexttemplate name="two"> <pdf:nextpage> <p>Hello, world!</p>"""
+
+    with tempfile.TemporaryFile() as pdf_file:
+        pisaDocument(
+            src=io.StringIO(HTML_CONTENT.format(head=css, extra_html=extra_html)),
+            dest=pdf_file
+        )
+        pdf_file.seek(0)
+        pdf_reader = PdfFileReader(pdf_file)
+
+        tools.assert_equal(pdf_reader.getNumPages(), 2)
+
+
 @skip_if(IN_PYPY, "This doesn't work in pypy")
 def test_document_creation_without_metadata():
     with tempfile.TemporaryFile() as pdf_file:
