@@ -142,7 +142,7 @@ def transform_attrs(obj, keys, container, func, extras=None):
     Allows to apply one function to set of keys cheching if key is in container,
     also trasform ccs key to report lab keys.
 
-    extras = Are extra params for func, it will be call like func(*[param1, param2]) 
+    extras = Are extra params for func, it will be call like func(*[param1, param2])
 
     obj = frag
     keys = [(reportlab, css), ... ]
@@ -167,7 +167,7 @@ def transform_attrs(obj, keys, container, func, extras=None):
 def copy_attrs(obj1, obj2, attrs):
     """
     Allows copy a list of attributes from object2 to object1.
-    Useful for copy ccs attributes to fragment  
+    Useful for copy ccs attributes to fragment
     """
     for attr in attrs:
         value = getattr(obj2, attr) if hasattr(obj2, attr) else None
@@ -178,7 +178,7 @@ def copy_attrs(obj1, obj2, attrs):
 
 def set_value(obj, attrs, value, _copy=False):
     """
-    Allows set the same value to a list of attributes 
+    Allows set the same value to a list of attributes
     """
     for attr in attrs:
         if _copy:
@@ -300,7 +300,10 @@ def getSize(value, relative=0, base=None, default=0.0):
         elif value in ("none", "0", "auto"):
             return 0.0
         elif relative:
-            if value[-2:] == 'em':  # XXX
+            if value[-3:] == 'rem':  # XXX
+                # 1rem = 1 * fontSize
+                return float(value[:-3].strip()) * relative
+            elif value[-2:] == 'em':  # XXX
                 # 1em = 1 * fontSize
                 return float(value[:-2].strip()) * relative
             elif value[-2:] == 'ex':  # XXX
@@ -583,6 +586,7 @@ class pisaFileObject:
     """
 
     def __init__(self, uri, basepath=None):
+
         self.basepath = basepath
         self.mimetype = None
         self.file = None
@@ -591,7 +595,7 @@ class pisaFileObject:
         self.local = None
         self.tmp_file = None
         uri = uri or str()
-        if type(uri) != str:
+        if not isinstance(uri, str):
             uri = uri.decode("utf-8")
         log.debug("FileObject %r, Basepath: %r", uri, basepath)
 
@@ -599,7 +603,18 @@ class pisaFileObject:
         if uri.startswith("data:"):
             m = _rx_datauri.match(uri)
             self.mimetype = m.group("mime")
-            b64 = urllib_unquote(m.group("data")).encode("utf-8")
+
+            b64 = urllib_unquote(m.group("data"))
+
+            # The data may be incorrectly unescaped... repairs needed
+            b64 = b64.strip("b'").strip("'").encode()
+            b64 = re.sub(b"\\n", b'', b64)
+            b64 = re.sub(b'[^A-Za-z0-9\+\/]+', b'', b64)
+
+            # Add padding as needed, to make length into a multiple of 4
+            #
+            b64 += b"=" * ((4 - len(b64) % 4) % 4)
+
             self.data = base64.b64decode(b64)
 
         else:
@@ -636,7 +651,10 @@ class pisaFileObject:
                 #mimetype = getMimeType(path)
 
                 # Using HTTPLIB
-                server, path = urllib2.splithost(uri[uri.find("//"):])
+                url_splitted = urlparse.urlsplit(uri)
+                server = url_splitted[1]
+                path = url_splitted[2]
+                path += "?" + url_splitted[3] if url_splitted[3] else ""
                 if uri.startswith("https://"):
                     conn = httplib.HTTPSConnection(server,  **httpConfig)
                 else:
@@ -653,7 +671,7 @@ class pisaFileObject:
                         import gzip
 
                         self.file = gzip.GzipFile(
-                            mode="rb", fileobj=six.StringIO(r1.read()))
+                            mode="rb", fileobj=six.BytesIO(r1.read()))
                     else:
                         self.file = pisaTempFile(r1.read())
                 else:
@@ -675,6 +693,8 @@ class pisaFileObject:
 
                 # Local data
                 if basepath:
+                    if sys.platform == 'win32' and os.path.isfile(basepath):
+                        basepath = os.path.dirname(basepath)
                     uri = os.path.normpath(os.path.join(basepath, uri))
 
                 if os.path.isfile(uri):
